@@ -53,6 +53,7 @@
 
 #include <memory>
 #include <macros.h>
+#include <drc/drc_item.h>
 #include "kiface_base.h"
 #include "pcbnew_settings.h"
 
@@ -325,6 +326,28 @@ bool PAD::FlashLayer( int aLayer, bool aOnlyCheckIfPermitted ) const
 }
 
 
+void PAD::SetDrillSizeX( const int aX )
+{
+    m_drill.x = aX;
+
+    if( GetDrillShape() == PAD_DRILL_SHAPE_CIRCLE )
+        SetDrillSizeY( aX );
+
+    SetDirty();
+}
+
+
+void PAD::SetDrillShape( PAD_DRILL_SHAPE_T aShape )
+{
+    m_drillShape = aShape;
+
+    if( aShape == PAD_DRILL_SHAPE_CIRCLE )
+        SetDrillSizeY( GetDrillSizeX() );
+
+    m_shapesDirty = true;
+}
+
+
 int PAD::GetRoundRectCornerRadius() const
 {
     return KiROUND( std::min( m_size.x, m_size.y ) * m_roundedCornerScale );
@@ -649,10 +672,45 @@ const BOX2I PAD::GetBoundingBox() const
 
 void PAD::SetAttribute( PAD_ATTRIB aAttribute )
 {
-    m_attribute = aAttribute;
+    if( m_attribute != aAttribute )
+    {
+        m_attribute = aAttribute;
 
-    if( aAttribute == PAD_ATTRIB::SMD )
-        m_drill = VECTOR2I( 0, 0 );
+        switch( aAttribute )
+        {
+        case PAD_ATTRIB::PTH:
+            // Plump up to all copper layers
+            m_layerMask |= LSET::AllCuMask();
+            break;
+
+        case PAD_ATTRIB::SMD:
+        case PAD_ATTRIB::CONN:
+        {
+            // Trim down to no more than one copper layer
+            LSET copperLayers = m_layerMask & LSET::AllCuMask();
+
+            if( copperLayers.count() > 1 )
+            {
+                m_layerMask &= ~LSET::AllCuMask();
+
+                if( copperLayers.test( B_Cu ) )
+                    m_layerMask.set( B_Cu );
+                else
+                    m_layerMask.set( copperLayers.Seq().front() );
+            }
+
+            // No hole
+            m_drill = VECTOR2I( 0, 0 );
+            break;
+        }
+
+        case PAD_ATTRIB::NPTH:
+            // No number; no net
+            m_number = wxEmptyString;
+            SetNetCode( NETINFO_LIST::UNCONNECTED );
+            break;
+        }
+    }
 
     SetDirty();
 }
@@ -684,7 +742,7 @@ void PAD::SetFPRelativeOrientation( const EDA_ANGLE& aAngle )
 }
 
 
-EDA_ANGLE PAD::GetFPRelativeOrientation()
+EDA_ANGLE PAD::GetFPRelativeOrientation() const
 {
     if( FOOTPRINT* parentFP = GetParentFootprint() )
         return GetOrientation() - parentFP->GetOrientation();
@@ -1690,6 +1748,397 @@ void PAD::TransformShapeToPolygon( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer,
 }
 
 
+std::vector<PCB_SHAPE*> PAD::Recombine( bool aIsDryRun, int maxError )
+{
+    FOOTPRINT* footprint = GetParentFootprint();
+
+    for( BOARD_ITEM* item : footprint->GraphicalItems() )
+        item->ClearFlags( SKIP_STRUCT );
+
+    auto findNext =
+            [&]( PCB_LAYER_ID aLayer ) -> PCB_SHAPE*
+            {
+                SHAPE_POLY_SET padPoly;
+                TransformShapeToPolygon( padPoly, aLayer, 0, maxError, ERROR_INSIDE );
+
+                for( BOARD_ITEM* item : footprint->GraphicalItems() )
+                {
+                    PCB_SHAPE* shape = dynamic_cast<PCB_SHAPE*>( item );
+
+                    if( !shape || ( shape->GetFlags() & SKIP_STRUCT ) )
+                        continue;
+
+                    if( shape->GetLayer() != aLayer )
+                        continue;
+
+                    if( shape->IsProxyItem() )    // Pad number (and net name) box
+                        return shape;
+
+                    SHAPE_POLY_SET drawPoly;
+                    shape->TransformShapeToPolygon( drawPoly, aLayer, 0, maxError, ERROR_INSIDE );
+                    drawPoly.BooleanIntersection( padPoly, SHAPE_POLY_SET::PM_FAST );
+
+                    if( !drawPoly.IsEmpty() )
+                        return shape;
+                }
+
+                return nullptr;
+            };
+
+    auto findMatching =
+            [&]( PCB_SHAPE* aShape ) -> std::vector<PCB_SHAPE*>
+            {
+                std::vector<PCB_SHAPE*> matching;
+
+                for( BOARD_ITEM* item : footprint->GraphicalItems() )
+                {
+                    PCB_SHAPE* other = dynamic_cast<PCB_SHAPE*>( item );
+
+                    if( !other || ( other->GetFlags() & SKIP_STRUCT ) )
+                        continue;
+
+                    if( GetLayerSet().test( other->GetLayer() )
+                            && aShape->Compare( other ) == 0 )
+                    {
+                        matching.push_back( other );
+                    }
+                }
+
+                return matching;
+            };
+
+    PCB_LAYER_ID            layer;
+    std::vector<PCB_SHAPE*> mergedShapes;
+
+    if( IsOnLayer( F_Cu ) )
+        layer = F_Cu;
+    else if( IsOnLayer( B_Cu ) )
+        layer = B_Cu;
+    else
+        layer = *GetLayerSet().UIOrder();
+
+    // If there are intersecting items to combine, we need to first make sure the pad is a
+    // custom-shape pad.
+    if( !aIsDryRun && findNext( layer ) && GetShape() != PAD_SHAPE::CUSTOM )
+    {
+        if( GetShape() == PAD_SHAPE::CIRCLE || GetShape() == PAD_SHAPE::RECTANGLE )
+        {
+            // Use the existing pad as an anchor
+            SetAnchorPadShape( GetShape() );
+            SetShape( PAD_SHAPE::CUSTOM );
+        }
+        else
+        {
+            // Create a new circular anchor and convert existing pad to a polygon primitive
+            SHAPE_POLY_SET existingOutline;
+            TransformShapeToPolygon( existingOutline, layer, 0, maxError, ERROR_INSIDE );
+
+            int minExtent = std::min( GetSize().x, GetSize().y );
+            SetAnchorPadShape( PAD_SHAPE::CIRCLE );
+            SetSize( VECTOR2I( minExtent, minExtent ) );
+            SetShape( PAD_SHAPE::CUSTOM );
+
+            PCB_SHAPE* shape = new PCB_SHAPE( nullptr, SHAPE_T::POLY );
+            shape->SetFilled( true );
+            shape->SetStroke( STROKE_PARAMS( 0, LINE_STYLE::SOLID ) );
+            shape->SetPolyShape( existingOutline );
+            shape->Move( - ShapePos() );
+            shape->Rotate( VECTOR2I( 0, 0 ), - GetOrientation() );
+            AddPrimitive( shape );
+        }
+    }
+
+    while( PCB_SHAPE* fpShape = findNext( layer ) )
+    {
+        fpShape->SetFlags( SKIP_STRUCT );
+
+        mergedShapes.push_back( fpShape );
+
+        if( !aIsDryRun )
+        {
+            PCB_SHAPE* primitive = static_cast<PCB_SHAPE*>( fpShape->Duplicate() );
+
+            primitive->SetParent( nullptr );
+            primitive->Move( - ShapePos() );
+            primitive->Rotate( VECTOR2I( 0, 0 ), - GetOrientation() );
+
+            AddPrimitive( primitive );
+        }
+
+        // See if there are other shapes that match and mark them for delete.  (KiCad won't
+        // produce these, but old footprints from other vendors have them.)
+        for( PCB_SHAPE* other : findMatching( fpShape ) )
+        {
+            other->SetFlags( SKIP_STRUCT );
+            mergedShapes.push_back( other );
+        }
+    }
+
+    for( BOARD_ITEM* item : footprint->GraphicalItems() )
+        item->ClearFlags( SKIP_STRUCT );
+
+    if( !aIsDryRun )
+        ClearFlags( ENTERED );
+
+    return mergedShapes;
+}
+
+
+void PAD::CheckPad( UNITS_PROVIDER* aUnitsProvider,
+                    const std::function<void( int aErrorCode,
+                                              const wxString& aMsg )>& aErrorHandler ) const
+{
+    VECTOR2I pad_size = GetSize();
+    VECTOR2I drill_size = GetDrillSize();
+    wxString msg;
+
+    if( GetShape() == PAD_SHAPE::CUSTOM )
+        pad_size = GetBoundingBox().GetSize();
+    else if( pad_size.x <= 0 || ( pad_size.y <= 0 && GetShape() != PAD_SHAPE::CIRCLE ) )
+        aErrorHandler( DRCE_PADSTACK_INVALID, _( "(Pad must have a positive size)" ) );
+
+    // Test hole against pad shape
+    if( IsOnCopperLayer() && GetDrillSize().x > 0 )
+    {
+        // Ensure the drill size can be handled in next calculations.
+        // Use min size = 4 IU to be able to build a polygon from a hole shape
+        const int min_drill_size = 4;
+
+        if( GetDrillSizeX() <= min_drill_size || GetDrillSizeY() <= min_drill_size )
+        {
+            msg.Printf( _( "(PTH pad hole size must be larger than %s)" ),
+                        aUnitsProvider->StringFromValue( min_drill_size, true ) );
+            aErrorHandler( DRCE_PADSTACK_INVALID, msg );
+        }
+
+        int            maxError = GetBoard()->GetDesignSettings().m_MaxError;
+        SHAPE_POLY_SET padOutline;
+
+        TransformShapeToPolygon( padOutline, UNDEFINED_LAYER, 0, maxError, ERROR_INSIDE );
+
+        if( GetAttribute() == PAD_ATTRIB::PTH )
+        {
+            // Test if there is copper area outside hole
+            std::shared_ptr<SHAPE_SEGMENT> hole = GetEffectiveHoleShape();
+            SHAPE_POLY_SET                 holeOutline;
+
+            TransformOvalToPolygon( holeOutline, hole->GetSeg().A, hole->GetSeg().B,
+                                    hole->GetWidth(), ARC_HIGH_DEF, ERROR_OUTSIDE );
+
+            SHAPE_POLY_SET copper = padOutline;
+            copper.BooleanSubtract( holeOutline, SHAPE_POLY_SET::POLYGON_MODE::PM_FAST );
+
+            if( copper.IsEmpty() )
+            {
+                aErrorHandler( DRCE_PADSTACK, _( "(PTH pad hole leaves no copper)" ) );
+            }
+            else
+            {
+                // Test if the pad hole is fully inside the copper area
+                holeOutline.BooleanSubtract( padOutline, SHAPE_POLY_SET::POLYGON_MODE::PM_FAST );
+
+                if( !holeOutline.IsEmpty() )
+                    aErrorHandler( DRCE_PADSTACK, _( "(PTH pad hole non fully inside copper)" ) );
+            }
+        }
+        else
+        {
+            // Test only if the pad hole's centre is inside the copper area
+            if( !padOutline.Collide( GetPosition() ) )
+                aErrorHandler( DRCE_PADSTACK, _( "(pad hole not inside pad shape)" ) );
+        }
+    }
+
+    if( GetLocalClearance() < 0 )
+        aErrorHandler( DRCE_PADSTACK, _( "(negative local clearance values have no effect)" ) );
+
+    // Some pads need a negative solder mask clearance (mainly for BGA with small pads)
+    // However the negative solder mask clearance must not create negative mask size
+    // Therefore test for minimal acceptable negative value
+    std::optional<int> solderMaskMargin = GetLocalSolderMaskMargin();
+
+    if( solderMaskMargin.has_value() && solderMaskMargin.value() < 0 )
+    {
+        int absMargin = abs( solderMaskMargin.value() );
+
+        if( GetShape() == PAD_SHAPE::CUSTOM )
+        {
+            for( const std::shared_ptr<PCB_SHAPE>& shape : GetPrimitives() )
+            {
+                BOX2I shapeBBox = shape->GetBoundingBox();
+
+                if( absMargin > shapeBBox.GetWidth() || absMargin > shapeBBox.GetHeight() )
+                {
+                    aErrorHandler( DRCE_PADSTACK, _( "(negative solder mask clearance is larger "
+                                                     "than some shape primitives; results may be "
+                                                     "surprising)" ) );
+
+                    break;
+                }
+            }
+        }
+        else if( absMargin > pad_size.x || absMargin > pad_size.y )
+        {
+            aErrorHandler( DRCE_PADSTACK, _( "(negative solder mask clearance is larger than pad; "
+                                             "no solder mask will be generated)" ) );
+        }
+    }
+
+    // Some pads need a positive solder paste clearance (mainly for BGA with small pads)
+    // However, a positive value can create issues if the resulting shape is too big.
+    // (like a solder paste creating a solder paste area on a neighbor pad or on the solder mask)
+    // So we could ask for user to confirm the choice
+    // For now we just check for disappearing paste
+    wxSize paste_size;
+    int    paste_margin = GetLocalSolderPasteMargin();
+    double paste_ratio = GetLocalSolderPasteMarginRatio();
+
+    paste_size.x = pad_size.x + paste_margin + KiROUND( pad_size.x * paste_ratio );
+    paste_size.y = pad_size.y + paste_margin + KiROUND( pad_size.y * paste_ratio );
+
+    if( paste_size.x <= 0 || paste_size.y <= 0 )
+    {
+        aErrorHandler( DRCE_PADSTACK, _( "(negative solder paste margin is larger than pad; "
+                                         "no solder paste mask will be generated)" ) );
+    }
+
+    LSET padlayers_mask = GetLayerSet();
+
+    if( padlayers_mask == 0 )
+        aErrorHandler( DRCE_PADSTACK_INVALID, _( "(Pad has no layer)" ) );
+
+    if( GetAttribute() == PAD_ATTRIB::PTH && !IsOnCopperLayer() )
+        aErrorHandler( DRCE_PADSTACK, _( "(PTH pad has no copper layers)" ) );
+
+    if( !padlayers_mask[F_Cu] && !padlayers_mask[B_Cu] )
+    {
+        if( ( drill_size.x || drill_size.y ) && GetAttribute() != PAD_ATTRIB::NPTH )
+        {
+            aErrorHandler( DRCE_PADSTACK, _( "(plated through holes normally have a copper pad on "
+                                             "at least one layer)" ) );
+        }
+    }
+
+    switch( GetAttribute() )
+    {
+    case PAD_ATTRIB::NPTH:   // Not plated, but through hole, a hole is expected
+    case PAD_ATTRIB::PTH:    // Pad through hole, a hole is also expected
+        if( drill_size.x <= 0
+            || ( drill_size.y <= 0 && GetDrillShape() == PAD_DRILL_SHAPE_T::PAD_DRILL_SHAPE_OBLONG ) )
+        {
+            aErrorHandler( DRCE_PAD_TH_WITH_NO_HOLE, wxEmptyString );
+        }
+        break;
+
+    case PAD_ATTRIB::CONN:      // Connector pads are smd pads, just they do not have solder paste.
+        if( padlayers_mask[B_Paste] || padlayers_mask[F_Paste] )
+        {
+            aErrorHandler( DRCE_PADSTACK, _( "(connector pads normally have no solder paste; use a "
+                                             "SMD pad instead)" ) );
+        }
+        KI_FALLTHROUGH;
+
+    case PAD_ATTRIB::SMD:       // SMD and Connector pads (One external copper layer only)
+    {
+        if( drill_size.x > 0 || drill_size.y > 0 )
+            aErrorHandler( DRCE_PADSTACK_INVALID, _( "(SMD pad has a hole)" ) );
+
+        LSET innerlayers_mask = padlayers_mask & LSET::InternalCuMask();
+
+        if( IsOnLayer( F_Cu ) && IsOnLayer( B_Cu ) )
+        {
+            aErrorHandler( DRCE_PADSTACK, _( "(SMD pad has copper on both sides of the board)" ) );
+        }
+        else if( IsOnLayer( F_Cu ) )
+        {
+            if( IsOnLayer( B_Mask ) )
+            {
+                aErrorHandler( DRCE_PADSTACK, _( "(SMD pad has copper and mask layers on different "
+                                                 "sides of the board)" ) );
+            }
+            else if( IsOnLayer( B_Paste ) )
+            {
+                aErrorHandler( DRCE_PADSTACK, _( "(SMD pad has copper and paste layers on different "
+                                                 "sides of the board)" ) );
+            }
+        }
+        else if( IsOnLayer( B_Cu ) )
+        {
+            if( IsOnLayer( F_Mask ) )
+            {
+                aErrorHandler( DRCE_PADSTACK, _( "(SMD pad has copper and mask layers on different "
+                                                 "sides of the board)" ) );
+            }
+            else if( IsOnLayer( F_Paste ) )
+            {
+                aErrorHandler( DRCE_PADSTACK, _( "(SMD pad has copper and paste layers on different "
+                                                 "sides of the board)" ) );
+            }
+        }
+        else if( innerlayers_mask.count() != 0 )
+        {
+            aErrorHandler( DRCE_PADSTACK, _( "(SMD pad has no outer layers)" ) );
+        }
+
+        break;
+    }
+    }
+
+    if( ( GetProperty() == PAD_PROP::FIDUCIAL_GLBL || GetProperty() == PAD_PROP::FIDUCIAL_LOCAL )
+            && GetAttribute() == PAD_ATTRIB::NPTH )
+    {
+        aErrorHandler( DRCE_PADSTACK, _( "('fiducial' property makes no sense on NPTH pads)" ) );
+    }
+
+    if( GetProperty() == PAD_PROP::TESTPOINT && GetAttribute() == PAD_ATTRIB::NPTH )
+        aErrorHandler( DRCE_PADSTACK, _( "('testpoint' property makes no sense on NPTH pads)" ) );
+
+    if( GetProperty() == PAD_PROP::HEATSINK && GetAttribute() == PAD_ATTRIB::NPTH )
+        aErrorHandler( DRCE_PADSTACK, _( "('heatsink' property makes no sense of NPTH pads)" ) );
+
+    if( GetProperty() == PAD_PROP::CASTELLATED && GetAttribute() != PAD_ATTRIB::PTH )
+        aErrorHandler( DRCE_PADSTACK, _( "('castellated' property is for PTH pads)" ) );
+
+    if( GetProperty() == PAD_PROP::BGA && GetAttribute() != PAD_ATTRIB::SMD )
+        aErrorHandler( DRCE_PADSTACK, _( "('BGA' property is for SMD pads)" ) );
+
+    if( GetShape() == PAD_SHAPE::ROUNDRECT )
+    {
+        if( GetRoundRectRadiusRatio() < 0.0 )
+            aErrorHandler( DRCE_PADSTACK_INVALID, _( "(negative corner radius is not allowed)" ) );
+        else if( GetRoundRectRadiusRatio() > 50.0 )
+            aErrorHandler( DRCE_PADSTACK, _( "(corner size will make pad circular)" ) );
+    }
+    else if( GetShape() == PAD_SHAPE::CHAMFERED_RECT )
+    {
+        if( GetChamferRectRatio() < 0.0 )
+            aErrorHandler( DRCE_PADSTACK_INVALID, _( "(negative corner chamfer is not allowed)" ) );
+        else if( GetChamferRectRatio() > 50.0 )
+            aErrorHandler( DRCE_PADSTACK_INVALID, _( "(corner chamfer is too large)" ) );
+    }
+    else if( GetShape() == PAD_SHAPE::TRAPEZOID )
+    {
+        if(    ( GetDelta().x < 0 && GetDelta().x < -GetSize().y )
+            || ( GetDelta().x > 0 && GetDelta().x > GetSize().y )
+            || ( GetDelta().y < 0 && GetDelta().y < -GetSize().x )
+            || ( GetDelta().y > 0 && GetDelta().y > GetSize().x ) )
+        {
+            aErrorHandler( DRCE_PADSTACK_INVALID, _( "(trapazoid delta is too large)" ) );
+        }
+    }
+
+    // PADSTACKS TODO: this will need to check each layer in the pad...
+    if( GetShape() == PAD_SHAPE::CUSTOM )
+    {
+        SHAPE_POLY_SET mergedPolygon;
+        MergePrimitivesAsPolygon( &mergedPolygon );
+
+        if( mergedPolygon.OutlineCount() > 1 )
+            aErrorHandler( DRCE_PADSTACK_INVALID, _( "(custom pad shape must resolve to a single polygon)" ) );
+    }
+}
+
+
 bool PAD::operator==( const BOARD_ITEM& aOther ) const
 {
     if( Type() != aOther.Type() )
@@ -1898,6 +2347,10 @@ static struct PAD_DESC
                 .Map( PAD_PROP::HEATSINK,          _HKI( "Heatsink pad" ) )
                 .Map( PAD_PROP::CASTELLATED,       _HKI( "Castellated pad" ) );
 
+        ENUM_MAP<PAD_DRILL_SHAPE_T>::Instance()
+                .Map( PAD_DRILL_SHAPE_CIRCLE,     _HKI( "Round" ) )
+                .Map( PAD_DRILL_SHAPE_OBLONG,     _HKI( "Oblong" ) );
+
         ENUM_MAP<ZONE_CONNECTION>& zcMap = ENUM_MAP<ZONE_CONNECTION>::Instance();
 
         if( zcMap.Choices().GetCount() == 0 )
@@ -1996,6 +2449,10 @@ static struct PAD_DESC
                     } );
         propMgr.AddProperty( roundRadiusRatio, groupPad );
 
+        propMgr.AddProperty( new PROPERTY_ENUM<PAD, PAD_DRILL_SHAPE_T>( _HKI( "Hole Shape" ),
+                    &PAD::SetDrillShape, &PAD::GetDrillShape ), groupPad )
+                .SetWriteableFunc( padCanHaveHole );
+
         propMgr.AddProperty( new PROPERTY<PAD, int>( _HKI( "Hole Size X" ),
                     &PAD::SetDrillSizeX, &PAD::GetDrillSizeX,
                     PROPERTY_DISPLAY::PT_SIZE ), groupPad )
@@ -2006,7 +2463,16 @@ static struct PAD_DESC
                     &PAD::SetDrillSizeY, &PAD::GetDrillSizeY,
                     PROPERTY_DISPLAY::PT_SIZE ), groupPad )
                 .SetWriteableFunc( padCanHaveHole )
-                .SetValidator( PROPERTY_VALIDATORS::PositiveIntValidator );
+                .SetValidator( PROPERTY_VALIDATORS::PositiveIntValidator )
+                .SetAvailableFunc(
+                        [=]( INSPECTABLE* aItem ) -> bool
+                        {
+                            // Circle holes have no usable y-size
+                            if( PAD* pad = dynamic_cast<PAD*>( aItem ) )
+                                return pad->GetDrillShape() != PAD_DRILL_SHAPE_CIRCLE;
+
+                            return true;
+                        } );
 
         propMgr.AddProperty( new PROPERTY_ENUM<PAD, PAD_PROP>( _HKI( "Fabrication Property" ),
                     &PAD::SetProperty, &PAD::GetProperty ), groupPad );
@@ -2062,3 +2528,4 @@ static struct PAD_DESC
 ENUM_TO_WXANY( PAD_ATTRIB );
 ENUM_TO_WXANY( PAD_SHAPE );
 ENUM_TO_WXANY( PAD_PROP );
+ENUM_TO_WXANY( PAD_DRILL_SHAPE_T );
