@@ -646,7 +646,7 @@ bool PNS_KICAD_IFACE_BASE::inheritTrackWidth( PNS::ITEM* aItem, int* aInheritedW
 
 
 bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM* aStartItem,
-                                        PNS::NET_HANDLE aNet )
+                                        PNS::NET_HANDLE aNet, VECTOR2D aStartPosition )
 {
     BOARD_DESIGN_SETTINGS& bds = m_board->GetDesignSettings();
     PNS::CONSTRAINT        constraint;
@@ -658,10 +658,29 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
     aSizes.SetMinClearance( bds.m_MinClearance );
     aSizes.SetClearanceSource( _( "board minimum clearance" ) );
 
+    int      startAnchor = 0;
+    VECTOR2I startPosInt( aStartPosition.x, aStartPosition.y );
+
+    if( aStartItem && aStartItem->Kind() == PNS::ITEM::SEGMENT_T )
+    {
+        // Find the start anchor which is closest to the start mouse location
+        VECTOR2I anchor0 = aStartItem->Anchor( 0 );
+        VECTOR2I anchor1 = aStartItem->Anchor( 1 );
+
+        VECTOR2<double> diff0( anchor0.x - startPosInt.x, anchor0.y - startPosInt.y );
+        VECTOR2<double> diff1( anchor1.x - startPosInt.x, anchor1.y - startPosInt.y );
+
+        double anchor0Distance = diff0.EuclideanNorm();
+        double anchor1Distance = diff1.EuclideanNorm();
+
+        if( anchor1Distance < anchor0Distance )
+            startAnchor = 1;
+    }
+
     if( aStartItem )
     {
         PNS::SEGMENT dummyTrack;
-        dummyTrack.SetEnds( aStartItem->Anchor( 0 ), aStartItem->Anchor( 0 ) );
+        dummyTrack.SetEnds( aStartItem->Anchor( startAnchor ), aStartItem->Anchor( startAnchor ) );
         dummyTrack.SetLayer( ToLAYER_ID( m_startLayer ) );
         dummyTrack.SetNet( static_cast<NETINFO_ITEM*>( aStartItem->Net() ) );
 
@@ -691,7 +710,7 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
     if( !found && bds.UseNetClassTrack() && aStartItem )
     {
         PNS::SEGMENT dummyTrack;
-        dummyTrack.SetEnds( aStartItem->Anchor( 0 ), aStartItem->Anchor( 0 ) );
+        dummyTrack.SetEnds( aStartItem->Anchor( startAnchor ), aStartItem->Anchor( startAnchor ) );
         dummyTrack.SetLayer( ToLAYER_ID( m_startLayer ) );
         dummyTrack.SetNet( static_cast<NETINFO_ITEM*>( aStartItem->Net() ) );
 
